@@ -38,7 +38,6 @@ import java.time.format.FormatStyle;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Enumeration;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
@@ -85,6 +84,7 @@ import org.apache.jmeter.samplers.SampleEvent;
 import org.apache.jmeter.save.SaveService;
 import org.apache.jmeter.services.FileServer;
 import org.apache.jmeter.testelement.TestElement;
+import org.apache.jmeter.testelement.TestElementSchema;
 import org.apache.jmeter.testelement.TestStateListener;
 import org.apache.jmeter.threads.RemoteThreadsListenerTestElement;
 import org.apache.jmeter.util.BeanShellInterpreter;
@@ -427,6 +427,29 @@ public class JMeter implements JMeterPlugin {
             JMeterUtils.setProperty("START.YMD", getFormatter("yyyyMMdd").format(now));// $NON-NLS-1$ $NON-NLS-2$
             JMeterUtils.setProperty("START.HMS", getFormatter("HHmmss").format(now));// $NON-NLS-1$ $NON-NLS-2$
 
+            // For unknown reason, TestElementSchema might fail to initialize in remote execution mode
+            // It reproduces with Java 11.0.13, and the error is StackOverflowError with the following stacktrace
+            // The workaround is to initialize Kotlin reflection before deserializing the test plan.
+            //  at java.security.SecureClassLoader.defineClass(SecureClassLoader.java:174) ~[?:?]
+            //  at java.net.URLClassLoader.defineClass(URLClassLoader.java:555) ~[?:?]
+            //  at java.net.URLClassLoader$1.run(URLClassLoader.java:458) ~[?:?]
+            //  at java.net.URLClassLoader$1.run(URLClassLoader.java:452) ~[?:?]
+            //  at java.security.AccessController.doPrivileged(Native Method) ~[?:?]
+            //  at java.net.URLClassLoader.findClass(URLClassLoader.java:451) ~[?:?]
+            //  at java.lang.ClassLoader.loadClass(ClassLoader.java:589) ~[?:?]
+            //  at org.apache.jmeter.DynamicClassLoader.loadClass(DynamicClassLoader.java:81) ~[ApacheJMeter.jar:5.5.1-SNAPSHOT]
+            //  at java.lang.ClassLoader.loadClass(ClassLoader.java:522) ~[?:?]
+            //  at kotlin.jvm.internal.ClassReference.<clinit>(ClassReference.kt:156) ~[kotlin-stdlib-1.8.21.jar:1.8.21-release-380(1.8.21)]
+            //  at kotlin.jvm.internal.ReflectionFactory.getOrCreateKotlinClass(ReflectionFactory.java:30) ~[kotlin-stdlib-1.8.21.jar)]
+            //  at kotlin.jvm.internal.Reflection.getOrCreateKotlinClass(Reflection.java:60) ~[kotlin-stdlib-1.8.21.jar:1.8.21-release-380(1.8.21)]
+            //  at org.apache.jmeter.testelement.TestElementSchema.<init>(TestElementSchema.kt:33) ~[ApacheJMeter_core.jar:5.5.1-SNAPSHOT]
+            //  at org.apache.jmeter.testelement.TestElementSchema$INSTANCE.<init>(TestElementSchema.kt:26) ~[ApacheJMeter_core.jar:5.5.1-SNAPSHOT]
+            //  at org.apache.jmeter.testelement.TestElementSchema$INSTANCE.<init>(TestElementSchema.kt) ~[ApacheJMeter_core.jar:5.5.1-SNAPSHOT]
+            //  at org.apache.jmeter.testelement.TestElementSchema.<clinit>(TestElementSchema.kt) ~[ApacheJMeter_core.jar:5.5.1-SNAPSHOT]
+            //  at org.apache.jmeter.protocol.java.sampler.BeanShellSampler.<clinit>(BeanShellSampler.java:53) ~[ApacheJMeter_java.jar:5.5.1-SNAPSHOT]
+            //  at jdk.internal.misc.Unsafe.ensureClassInitialized0(Native Method) ~[?:?]
+            TestElementSchema.INSTANCE.getGuiClass();
+
             if (parser.getArgumentById(VERSION_OPT) != null) {
                 displayAsciiArt();
             } else if (parser.getArgumentById(HELP_OPT) != null) {
@@ -509,7 +532,7 @@ public class JMeter implements JMeterPlugin {
      * @param deleteReportFolder true means delete report folder
      * @throws IllegalArgumentException
      */
-    private void extractAndSetReportOutputFolder(CLArgsParser parser, boolean deleteReportFolder) {
+    private static void extractAndSetReportOutputFolder(CLArgsParser parser, boolean deleteReportFolder) {
         CLOption reportOutputFolderOpt = parser
                 .getArgumentById(REPORT_OUTPUT_FOLDER_OPT);
         if(reportOutputFolderOpt != null) {
@@ -526,7 +549,7 @@ public class JMeter implements JMeterPlugin {
     /**
      * Displays as ASCII Art Apache JMeter version + Copyright notice
      */
-    private void displayAsciiArt() {
+    private static void displayAsciiArt() {
         try (InputStream inputStream = JMeter.class.getResourceAsStream("jmeter_as_ascii_art.txt")) {
             if(inputStream != null) {
                 String text = IOUtils.toString(inputStream, StandardCharsets.UTF_8);
@@ -539,13 +562,13 @@ public class JMeter implements JMeterPlugin {
     }
 
     // Update classloader if necessary
-    private void updateClassLoader() throws MalformedURLException {
+    private static void updateClassLoader() throws MalformedURLException {
         updatePath("search_paths",";", true); //$NON-NLS-1$//$NON-NLS-2$
         updatePath("user.classpath",File.pathSeparator, true);//$NON-NLS-1$
         updatePath("plugin_dependency_paths",";", false);//$NON-NLS-1$
     }
 
-    private void updatePath(String property, String sep, boolean cp) throws MalformedURLException {
+    private static void updatePath(String property, String sep, boolean cp) throws MalformedURLException {
         String userpath= JMeterUtils.getPropDefault(property,"");// $NON-NLS-1$
         if (userpath.length() <= 0) {
             return;
@@ -572,7 +595,7 @@ public class JMeter implements JMeterPlugin {
     /**
      *
      */
-    private void startOptionalServers() {
+    private static void startOptionalServers() {
         int bshport = JMeterUtils.getPropDefault("beanshell.server.port", 0);// $NON-NLS-1$
         String bshfile = JMeterUtils.getPropDefault("beanshell.server.file", "");// $NON-NLS-1$ $NON-NLS-2$
         if (bshport > 0) {
@@ -601,7 +624,7 @@ public class JMeter implements JMeterPlugin {
     /**
      * Runs user configured init scripts
      */
-    void runInitScripts() {
+    static void runInitScripts() {
         // Should we run a beanshell script on startup?
         String bshinit = JMeterUtils.getProperty("beanshell.init.file");// $NON-NLS-1$
         if (bshinit != null){
@@ -651,7 +674,7 @@ public class JMeter implements JMeterPlugin {
     }
 
 
-    private Map<String, List<String>> getEnginesAndExtensions(ScriptEngineManager scriptEngineManager) {
+    private static Map<String, List<String>> getEnginesAndExtensions(ScriptEngineManager scriptEngineManager) {
         return scriptEngineManager.getEngineFactories().stream()
                 .collect(Collectors.toMap(
                         f -> f.getLanguageName() + " (" + f.getLanguageVersion() + ")",
@@ -662,7 +685,7 @@ public class JMeter implements JMeterPlugin {
      * Sets a proxy server for the JVM if the command line arguments are
      * specified.
      */
-    private void setProxy(CLArgsParser parser) throws IllegalUserActionException {
+    private static void setProxy(CLArgsParser parser) throws IllegalUserActionException {
         if (parser.getArgumentById(PROXY_USERNAME) != null) {
             Properties jmeterProps = JMeterUtils.getJMeterProperties();
             if (parser.getArgumentById(PROXY_PASSWORD) != null) {
@@ -880,7 +903,7 @@ public class JMeter implements JMeterPlugin {
      * Checks for LAST or LASTsuffix.
      * Returns the LAST name with .JMX replaced by suffix.
      */
-    private String processLAST(final String jmlogfile, final String suffix) {
+    private static String processLAST(final String jmlogfile, final String suffix) {
         if (USE_LAST_JMX.equals(jmlogfile) || USE_LAST_JMX.concat(suffix).equals(jmlogfile)){
             String last = LoadRecentProject.getRecentFile(0);// most recent
             if (last.toUpperCase(Locale.ENGLISH).endsWith(JMX_SUFFIX)){
@@ -971,13 +994,11 @@ public class JMeter implements JMeterPlugin {
             if (deleteResultFile) {
                 SearchByClass<ResultCollector> resultListeners = new SearchByClass<>(ResultCollector.class);
                 clonedTree.traverse(resultListeners);
-                Iterator<ResultCollector> irc = resultListeners.getSearchResults().iterator();
-                while (irc.hasNext()) {
-                    ResultCollector rc = irc.next();
+                for (ResultCollector rc : resultListeners.getSearchResults()) {
                     File resultFile = new File(rc.getFilename());
                     if (resultFile.exists() && !resultFile.delete()) {
                         throw new IllegalStateException("Could not delete results file " + resultFile.getAbsolutePath()
-                            + "(canRead:"+resultFile.canRead()+", canWrite:"+resultFile.canWrite()+")");
+                                + "(canRead:" + resultFile.canRead() + ", canWrite:" + resultFile.canWrite() + ")");
                     }
                 }
             }
@@ -1170,13 +1191,13 @@ public class JMeter implements JMeterPlugin {
 
         private AtomicInteger startedRemoteEngines = new AtomicInteger(0);
 
-        private ConcurrentLinkedQueue<JMeterEngine> remoteEngines = new ConcurrentLinkedQueue<>();
+        private final ConcurrentLinkedQueue<JMeterEngine> remoteEngines = new ConcurrentLinkedQueue<>();
 
         private final ReportGenerator reportGenerator;
 
-        private RunMode runMode;
+        private final RunMode runMode;
 
-        private boolean remoteStop;
+        private final boolean remoteStop;
 
         /**
          * Listener for remote test
@@ -1190,7 +1211,7 @@ public class JMeter implements JMeterPlugin {
             this.reportGenerator = reportGenerator;
         }
 
-        public void setStartedRemoteEngines(List<JMeterEngine> engines) {
+        public void setStartedRemoteEngines(List<? extends JMeterEngine> engines) {
             if (runMode != RunMode.REMOTE) {
                 throw new IllegalArgumentException("This method should only be called in RunMode.REMOTE");
             }
@@ -1276,7 +1297,7 @@ public class JMeter implements JMeterPlugin {
          * Runs daemon thread which waits a short while;
          * if JVM does not exit, lists remaining non-daemon threads on stdout.
          */
-        private void checkForRemainingThreads() {
+        private static void checkForRemainingThreads() {
             // This cannot be a JMeter class variable, because properties
             // are not initialised until later.
             final int pauseToCheckForRemainingThreads =
@@ -1358,7 +1379,7 @@ public class JMeter implements JMeterPlugin {
         return "true".equals(System.getProperty(JMeter.JMETER_NON_GUI)); //$NON-NLS-1$
     }
 
-    private static void startUdpDdaemon(final List<JMeterEngine> engines) {
+    private static void startUdpDdaemon(final List<? extends JMeterEngine> engines) {
         int port = JMeterUtils.getPropDefault("jmeterengine.nongui.port", UDP_PORT_DEFAULT); // $NON-NLS-1$
         int maxPort = JMeterUtils.getPropDefault("jmeterengine.nongui.maxport", 4455); // $NON-NLS-1$
         if (port > 1000){
@@ -1378,7 +1399,7 @@ public class JMeter implements JMeterPlugin {
         }
     }
 
-    private static void waitForSignals(final List<JMeterEngine> engines, DatagramSocket socket) {
+    private static void waitForSignals(final List<? extends JMeterEngine> engines, DatagramSocket socket) {
         byte[] buf = new byte[80];
         System.out.println("Waiting for possible Shutdown/StopTestNow/HeapDump/ThreadDump message on port "+socket.getLocalPort());//NOSONAR
         DatagramPacket request = new DatagramPacket(buf, buf.length);
