@@ -2,33 +2,36 @@
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
+ * The ASF licenses this file to you under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ * http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
  */
 
 package org.apache.jmeter.gui;
 
 import java.awt.Component;
+import java.awt.event.ActionEvent;
 import java.awt.event.MouseEvent;
 import java.beans.Introspector;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.IdentityHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.prefs.Preferences;
 
 import javax.swing.JCheckBoxMenuItem;
+import javax.swing.JComponent;
 import javax.swing.JOptionPane;
 import javax.swing.JPopupMenu;
 import javax.swing.JToolBar;
@@ -37,6 +40,8 @@ import javax.swing.SwingUtilities;
 import org.apache.jmeter.engine.util.ValueReplacer;
 import org.apache.jmeter.exceptions.IllegalUserActionException;
 import org.apache.jmeter.gui.UndoHistory.HistoryListener;
+import org.apache.jmeter.gui.action.ActionNames;
+import org.apache.jmeter.gui.action.ActionRouter;
 import org.apache.jmeter.gui.action.TreeNodeNamingPolicy;
 import org.apache.jmeter.gui.action.impl.DefaultTreeNodeNamingPolicy;
 import org.apache.jmeter.gui.logging.GuiLogEventBus;
@@ -56,6 +61,8 @@ import org.apache.jmeter.util.JMeterUtils;
 import org.apache.jmeter.util.LocaleChangeEvent;
 import org.apache.jmeter.util.LocaleChangeListener;
 import org.apache.jorphan.collections.HashTree;
+import org.apache.jorphan.gui.JFactory;
+import org.apiguardian.api.API;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -72,7 +79,9 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
 
     private static final Logger log = LoggerFactory.getLogger(GuiPackage.class);
 
-    private static final String SBR_PREFS_KEY = "save_before_run";
+    private static final String LAF_EPOCH = "JMeter.laf_epoch"; // $NON-NLS-1$
+
+    private static final String SBR_PREFS_KEY = "save_before_run"; // $NON-NLS-1$
 
     private static final String SAVE_BEFORE_RUN_PROPERTY = "save_automatically_before_run"; // $NON-NLS-1$
 
@@ -92,20 +101,27 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
     /**
      * Map from TestElement to JMeterGUIComponent, mapping the nodes in the tree
      * to their corresponding GUI components.
+     * <p>This enables to associate {@link UnsharedComponent} UIs with their {@link TestElement}.</p>
      */
-    private Map<TestElement, JMeterGUIComponent> nodesToGui = new HashMap<>();
+    private final IdentityHashMap<TestElement, JMeterGUIComponent> nodesToGui = new IdentityHashMap<>();
 
     /**
      * Map from Class to JMeterGUIComponent, mapping the Class of a GUI
      * component to an instance of that component.
      */
-    private Map<Class<?>, JMeterGUIComponent> guis = new HashMap<>();
+    private final Map<Class<?>, JMeterGUIComponent> guis = new HashMap<>();
 
     /**
      * Map from Class to TestBeanGUI, mapping the Class of a TestBean to an
      * instance of TestBeanGUI to be used to edit such components.
      */
-    private Map<Class<?>, JMeterGUIComponent> testBeanGUIs = new HashMap<>();
+    private final Map<Class<?>, JMeterGUIComponent> testBeanGUIs = new HashMap<>();
+
+    /**
+     * Tracks the number of times Look and Feel was changed.
+     * It enables partial invalidation of the cached UIs.
+     */
+    private final AtomicInteger lafEpoch = new AtomicInteger();
 
     /** The currently selected node in the tree. */
     private JMeterTreeNode currentNode = null;
@@ -134,13 +150,13 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
     private LoggerPanel loggerPanel;
 
     /** History for tree states */
-    private UndoHistory undoHistory = new UndoHistory();
+    private final UndoHistory undoHistory = new UndoHistory();
 
     /** GUI Logging Event Bus. */
-    private GuiLogEventBus logEventBus = new GuiLogEventBus();
+    private final GuiLogEventBus logEventBus = new GuiLogEventBus();
 
     /** Listeners for events on test plan */
-    private List<TestPlanListener> testPlanListeners = Collections.synchronizedList(new ArrayList<>());
+    private final List<TestPlanListener> testPlanListeners = Collections.synchronizedList(new ArrayList<>());
 
     /**
      * Private constructor to permit instantiation only from within this class.
@@ -221,7 +237,7 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
             }
             return getGui(node, guiClass, testClass);
         } catch (ClassNotFoundException e) {
-            log.error("Could not get GUI for " + node, e);
+            log.error("Could not get GUI for {}", node, e);
             return null;
         }
     }
@@ -250,6 +266,8 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
             if (comp == null) {
                 comp = getGuiFromCache(guiClass, testClass);
                 nodesToGui.put(node, comp);
+            } else {
+                updateUi(comp);
             }
             log.debug("Gui retrieved = {}", comp);
             return comp;
@@ -266,6 +284,7 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
      * @param node
      *            the test element being removed
      */
+    @API(since = "5.3", status = API.Status.MAINTAINED)
     public void removeNode(TestElement node) {
         nodesToGui.remove(node);
     }
@@ -356,7 +375,7 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
             nodesToGui.put(node, comp);
             return node;
         } catch (NoClassDefFoundError e) {
-            log.error("Problem retrieving gui for " + objClass, e);
+            log.error("Problem retrieving gui for {}", objClass, e);
             String msg="Cannot find class: "+e.getMessage();
             JOptionPane.showMessageDialog(null,
                     msg,
@@ -364,7 +383,7 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
                     JOptionPane.ERROR_MESSAGE);
             throw new RuntimeException(e.toString(), e); // Probably a missing jar
         } catch ( ReflectiveOperationException e) {
-            log.error("Problem retrieving gui for " + objClass, e);
+            log.error("Problem retrieving gui for {}", objClass, e);
             throw new RuntimeException(e.toString(), e); // Programming error: bail out.
         }
     }
@@ -408,9 +427,23 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
                 }
             }
         }
+        updateUi(comp);
         return comp;
     }
 
+    private void updateUi(JMeterGUIComponent comp) {
+        if (!(comp instanceof JComponent)) {
+            return;
+        }
+        JComponent jc = (JComponent) comp;
+        Object epoch = jc.getClientProperty(LAF_EPOCH);
+        int currentLafEpoch = lafEpoch.get();
+        if (epoch instanceof Integer && ((Integer) epoch) < currentLafEpoch) {
+            JFactory.updateUi(jc);
+        }
+        jc.putClientProperty(LAF_EPOCH, currentLafEpoch);
+    }
+
     /**
      * Update the GUI for the currently selected node. The GUI component is
      * configured to reflect the settings in the current tree node.
@@ -485,7 +518,7 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
     /**
      * The dirty property is a flag that indicates whether there are parts of
      * JMeter's test tree that the user has not saved since last modification.
-     * Various (@link Command actions) set this property when components are
+     * Various {@link org.apache.jmeter.gui.action.Command} actions set this property when components are
      * modified/created/saved.
      *
      * @param dirty
@@ -634,10 +667,9 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
         if (popup != null) {
             log.debug("Showing pop up for {} at x,y = {},{}", invoker, e.getX(), e.getY());
 
-            popup.pack();
+            // Enforce heavyweight popup to show shadows on macOS
+            popup.setLightWeightPopupEnabled(false);
             popup.show(invoker, e.getX(), e.getY());
-            popup.setVisible(true);
-            popup.requestFocusInWindow();
         }
     }
 
@@ -650,32 +682,39 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
         // will flush it away):
         updateCurrentNode();
 
-        // Forget about all GUIs we've created so far: we'll need to re-created
-        // them all!
-        guis = new HashMap<>();
-        nodesToGui = new HashMap<>();
-        testBeanGUIs = new HashMap<>();
-
-        // BeanInfo objects also contain locale-sensitive data -- flush them
-        // away:
         Introspector.flushCaches();
 
-        // Now put the current GUI in place. [This code was copied from the
-        // EditCommand action -- we can't just trigger the action because that
-        // would populate the current node with the contents of the new GUI --
-        // which is empty.]
-        MainFrame mf = getMainFrame(); // Fetch once
-        if (mf == null) { // Probably caused by unit testing on headless system
-            log.warn("Mainframe is null");
-        } else {
-            mf.setMainPanel((javax.swing.JComponent) getCurrentGui());
-            mf.setEditMenu(getTreeListener().getCurrentNode().createPopupMenu());
-        }
+        invalidateCachedUi();
+    }
+
+    public void invalidateCachedUi() {
+        // Save current edits from the GUI to tree model just in case
+        updateCurrentNode();
+
+        // Invalidate UIs
+        guis.clear();
+        nodesToGui.clear();
+        testBeanGUIs.clear();
+
+        // Call "start edit for the current tree node" action to show the UI
+        ActionRouter.getInstance().actionPerformed(
+                new ActionEvent(this, 3334, ActionNames.EDIT)
+        );
+    }
+
+    /**
+     * Tells hidden GUI components to update UIs when they are shown.
+     * <p>When Look and Feel (or zoom scaling) changes, only visible components are updated.
+     * The hidden ones are updated as they are shown.</p>
+     */
+    @API(since = "5.3", status = API.Status.EXPERIMENTAL)
+    public void updateUIForHiddenComponents() {
+        lafEpoch.getAndIncrement();
     }
 
     private String testPlanFile;
 
-    private final List<Stoppable> stoppables = Collections.synchronizedList(new ArrayList<Stoppable>());
+    private final List<Stoppable> stoppables = Collections.synchronizedList(new ArrayList<>());
 
     private TreeNodeNamingPolicy namingPolicy;
 
@@ -866,7 +905,7 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
      * @param el {@link TestElement}
      * @return int checksum
      */
-    private int getTestElementCheckSum(TestElement el) {
+    private static int getTestElementCheckSum(TestElement el) {
         int ret = el.getClass().hashCode();
         PropertyIterator it = el.propertyIterator();
         while (it.hasNext()) {
@@ -881,8 +920,8 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
                     ret ^= stringValue.hashCode();
                 } else {
                     if (log.isDebugEnabled()) {
-                        log.debug("obj.getStringValue() returned null for test element:"
-                                + el.getName() + " at property:" + obj.getName());
+                        log.debug("obj.getStringValue() returned null for test element:{} at property:{}",
+                                el.getName(), obj.getName());
                     }
                 }
             }
@@ -912,7 +951,7 @@ public final class GuiPackage implements LocaleChangeListener, HistoryListener {
                 this.namingPolicy = (TreeNodeNamingPolicy) implementationClass.getDeclaredConstructor().newInstance();
 
             } catch (Exception ex) {
-                log.error("Failed to create configured naming policy:" + namingPolicyImplementation + ", will use default one", ex);
+                log.error("Failed to create configured naming policy:{}, will use default one", namingPolicyImplementation, ex);
                 this.namingPolicy = new DefaultTreeNodeNamingPolicy();
             }
         }
