@@ -17,6 +17,13 @@
 
 package org.apache.jmeter.junit;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
+import static org.junit.jupiter.api.Assumptions.assumeFalse;
+
 import java.awt.Component;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -32,13 +39,17 @@ import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Properties;
+import java.util.Set;
+import java.util.function.Supplier;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import javax.swing.SwingUtilities;
 import javax.xml.parsers.DocumentBuilder;
@@ -47,22 +58,49 @@ import javax.xml.parsers.ParserConfigurationException;
 
 import org.apache.commons.lang3.StringUtils;
 import org.apache.jmeter.config.gui.ObsoleteGui;
+import org.apache.jmeter.control.IfControllerSchema;
+import org.apache.jmeter.control.LoopControllerSchema;
+import org.apache.jmeter.control.gui.TestFragmentControllerGui;
 import org.apache.jmeter.dsl.DslPrinterTraverser;
+import org.apache.jmeter.extractor.RegexExtractorSchema;
+import org.apache.jmeter.gui.GuiComponentHolder;
 import org.apache.jmeter.gui.JMeterGUIComponent;
+import org.apache.jmeter.gui.NamePanel;
 import org.apache.jmeter.gui.UnsharedComponent;
 import org.apache.jmeter.gui.tree.JMeterTreeNode;
 import org.apache.jmeter.loadsave.IsEnabledNormalizer;
+import org.apache.jmeter.protocol.http.control.gui.AjpSamplerGui;
+import org.apache.jmeter.protocol.http.control.gui.GraphQLHTTPSamplerGui;
+import org.apache.jmeter.protocol.http.sampler.HTTPSamplerBaseSchema;
+import org.apache.jmeter.protocol.java.config.gui.JavaConfigGui;
+import org.apache.jmeter.protocol.java.control.gui.JUnitTestSamplerGui;
+import org.apache.jmeter.protocol.java.control.gui.JavaTestSamplerGui;
+import org.apache.jmeter.protocol.jms.control.gui.JMSSamplerGui;
 import org.apache.jmeter.save.SaveService;
 import org.apache.jmeter.testbeans.TestBean;
 import org.apache.jmeter.testbeans.gui.TestBeanGUI;
+import org.apache.jmeter.testelement.AbstractScopedTestElementSchema;
 import org.apache.jmeter.testelement.TestElement;
+import org.apache.jmeter.testelement.TestElementSchema;
 import org.apache.jmeter.testelement.property.JMeterProperty;
 import org.apache.jmeter.testelement.property.PropertyIterator;
+import org.apache.jmeter.testelement.schema.CollectionPropertyDescriptor;
+import org.apache.jmeter.testelement.schema.PropertyDescriptor;
+import org.apache.jmeter.testelement.schema.TestElementPropertyDescriptor;
+import org.apache.jmeter.threads.ThreadGroupSchema;
+import org.apache.jmeter.threads.gui.PostThreadGroupGui;
+import org.apache.jmeter.threads.gui.SetupThreadGroupGui;
+import org.apache.jmeter.threads.openmodel.gui.OpenModelThreadGroupGui;
 import org.apache.jmeter.util.JMeterUtils;
+import org.apache.jmeter.visualizers.backend.BackendListenerGui;
 import org.apache.jorphan.reflect.ClassFinder;
 import org.apache.jorphan.util.JOrphanUtils;
-import org.junit.runner.Describable;
-import org.junit.runner.Description;
+import org.junit.jupiter.api.AfterAll;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.TestInstance;
+import org.junit.jupiter.api.parallel.Isolated;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.MethodSource;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.w3c.dom.Document;
@@ -70,10 +108,9 @@ import org.w3c.dom.NodeList;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 
-import junit.framework.Test;
-import junit.framework.TestSuite;
-
-public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
+@Isolated("changes default locale")
+@TestInstance(TestInstance.Lifecycle.PER_CLASS) // maps must persist across test method executions
+public class JMeterTest extends JMeterTestCase {
     private static final Logger log = LoggerFactory.getLogger(JMeterTest.class);
 
     private static Map<String, Boolean> guiTitles;
@@ -86,80 +123,17 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
 
     private static final Locale DEFAULT_LOCALE = Locale.getDefault();
 
-    public JMeterTest(String name) {
-        super(name);
-    }
-
-    /*
-     * The suite() method creates separate test suites for each of the types of
-     * test. The suitexxx() methods create a list of items to be tested, and
-     * create a new test instance for each.
-     *
-     * Each test type has its own constructor, which saves the item to be tested
-     *
-     * Note that the suite() method must be static, and the methods to run the
-     * tests must be instance methods so that they can pick up the item value
-     * which was saved by the constructor.
-     *
-     */
-
-    // Constructor for Serializable tests
-    private Serializable serObj;
-
-    public JMeterTest(String testName, Serializable ser) {
-        super(testName);// Save the method name
-        serObj = ser;
-    }
-
-    // Constructor for GUI tests
-    private JMeterGUIComponent guiItem;
-
-    public JMeterTest(String testName, JMeterGUIComponent gc) {
-        super(testName);// Save the method name
-        guiItem = gc;
-    }
-
-    @Override
-    public Description getDescription() {
-        StringBuilder sb = new StringBuilder();
-        sb.append(getName());
-        if (guiItem instanceof TestBeanGUI) {
-            sb.append(" ").append(guiItem);
-        } else if (guiItem != null) {
-            sb.append(" ").append(guiItem.getClass().getName());
-        } else if (serObj != null) {
-            sb.append(" ").append(serObj.getClass().getName());
-        }
-        return Description.createTestDescription(getClass(), sb.toString());
-    }
-
     private static volatile boolean classPathShown = false;// Only show classpath once
 
-    /*
-     * Use a suite to allow the tests to be generated at run-time
-     */
-    public static Test suite() throws Throwable {
-        TestSuite suite = new TestSuite("JMeterTest");
-
-        // The Locale used to instantiate the GUI objects
+    @BeforeAll
+    public static void setLocale() {
         JMeterUtils.setLocale(TEST_LOCALE);
         Locale.setDefault(TEST_LOCALE);
-        // Needs to be done before any GUI classes are instantiated
-
-        suite.addTest(new JMeterTest("readAliases"));
-        suite.addTest(new JMeterTest("createTitleSet"));
-        suite.addTest(new JMeterTest("createTagSet"));
-        suite.addTest(suiteGUIComponents());
-        suite.addTest(suiteSerializableElements());
-        suite.addTest(suiteBeanComponents());
-        suite.addTest(new JMeterTest("checkGuiSet"));
-
-        suite.addTest(new JMeterTest("resetLocale")); // revert
-        return suite;
     }
 
     // Restore the original Locale
-    public void resetLocale(){
+    @AfterAll
+    public static void resetLocale() {
         JMeterUtils.setLocale(DEFAULT_LOCALE);
         Locale.setDefault(DEFAULT_LOCALE);
     }
@@ -167,7 +141,8 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
     /*
      * Extract titles from component_reference.xml
      */
-    public void createTitleSet() throws Exception {
+    @BeforeAll
+    public static void createTitleSet() throws Exception {
         guiTitles = new HashMap<>(90);
 
         String compref = "../xdocs/usermanual/component_reference.xml";
@@ -197,7 +172,7 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
      * @throws IOException when stream can not be read
      * @throws SAXException in case of XML parsing error
      */
-    private org.w3c.dom.Element getBodyFromXMLDocument(InputStream stream)
+    private static org.w3c.dom.Element getBodyFromXMLDocument(InputStream stream)
             throws ParserConfigurationException, SAXException, IOException {
         DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
         dbf.setIgnoringElementContentWhitespace(true);
@@ -212,7 +187,8 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
     /*
      * Extract titles from component_reference.xml
      */
-    public void createTagSet() throws Exception {
+    @BeforeAll
+    public static void createTagSet() throws Exception {
         guiTags = new HashMap<>(90);
 
         String compref = "../xdocs/usermanual/component_reference.xml";
@@ -243,26 +219,26 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
                 .collect(Collectors.toList());
     }
 
-    public void checkGuiSet() {
+    @AfterAll
+    public static void checkGuiSet() {
         guiTitles.remove("Example Sampler");// We don't mind if this is left over
         guiTitles.remove("Sample_Result_Save_Configuration");// Ditto, not a sampler
         assertEquals(
-                "Should not have any names left over in guiTitles map, check name of components in EN (default) Locale, "
+                "[]",
+                keysWithFalseValues(guiTitles).toString(),
+                () -> "Should not have any names left over in guiTitles map, check name of components in EN (default) Locale, "
                         + "which must match name attribute of component, check java.awt.HeadlessException errors before,"
                         + " we are running with '-Djava.awt.headless="
-                        + System.getProperty("java.awt.headless") + "'",
-                "[]",
-                keysWithFalseValues(guiTitles).toString()
-        );
+                        + System.getProperty("java.awt.headless") + "'");
     }
 
-    /*
-     * Test GUI elements - create the suite of tests
-     */
-    private static Test suiteGUIComponents() throws Throwable {
-        TestSuite suite = new TestSuite("GuiComponents");
+    static Collection<GuiComponentHolder> customGuiComponents() throws Throwable {
+        List<GuiComponentHolder> components = new ArrayList<>();
         for (Object o : getObjects(JMeterGUIComponent.class)) {
             JMeterGUIComponent item = (JMeterGUIComponent) o;
+            if (item.getClass() == TestBeanGUI.class) {
+                continue;
+            }
             if (item instanceof JMeterTreeNode) {
                 System.out.println("o.a.j.junit.JMeterTest INFO: JMeterGUIComponent: skipping all tests  " + item.getClass().getName());
                 continue;
@@ -270,49 +246,31 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
             if (item instanceof ObsoleteGui) {
                 continue;
             }
-            TestSuite ts = new TestSuite(item.getClass().getName());
-            ts.addTest(new JMeterTest("GUIComponents1", item));
-            if (item instanceof TestBeanGUI) {
-                System.out.println("o.a.j.junit.JMeterTest INFO: JMeterGUIComponent: skipping some tests " + item.getClass().getName());
-            } else {
-                ts.addTest(new JMeterTest("GUIComponents2", item));
-                ts.addTest(new JMeterTest("saveLoadShouldKeepElementIntact", item));
-                ts.addTest(new JMeterTest("propertiesShouldNotBeInitializedToNullValues", item));
-                ts.addTest(new JMeterTest("runGUITitle", item));
-            }
-            suite.addTest(ts);
+            components.add(new GuiComponentHolder(item));
         }
-        return suite;
+        return components;
     }
 
-
     /*
      * Test GUI elements - create the suite of tests
      */
-    private static Test suiteBeanComponents() throws Throwable {
-        TestSuite suite = new TestSuite("BeanComponents");
+    static Collection<GuiComponentHolder> guiComponents() throws Throwable {
+        List<GuiComponentHolder> components = new ArrayList<>(customGuiComponents());
         for (Object o : getObjects(TestBean.class)) {
             Class<?> c = o.getClass();
-            try {
-                JMeterGUIComponent item = new TestBeanGUI(c);
-                TestSuite ts = new TestSuite(item.getClass().getName());
-                ts.addTest(new JMeterTest("GUIComponents2", item));
-                ts.addTest(new JMeterTest("saveLoadShouldKeepElementIntact", item));
-                ts.addTest(new JMeterTest("propertiesShouldNotBeInitializedToNullValues", item));
-                ts.addTest(new JMeterTest("runGUITitle", item));
-                suite.addTest(ts);
-            } catch (IllegalArgumentException e) {
-                System.out.println("o.a.j.junit.JMeterTest Cannot create test for " + c.getName() + " " + e);
-                e.printStackTrace(System.out);
-            }
+            JMeterGUIComponent item = new TestBeanGUI(c);
+            components.add(new GuiComponentHolder(item));
         }
-        return suite;
+        return components;
     }
 
     /*
      * Test GUI elements - run the test
      */
-    public void runGUITitle() throws Exception {
+    @ParameterizedTest
+    @MethodSource("guiComponents")
+    public void runGUITitle(GuiComponentHolder componentHolder) throws Exception {
+        JMeterGUIComponent guiItem = componentHolder.getComponent();
         if (!guiTitles.isEmpty()) {
             String title = guiItem.getDocAnchor();
             boolean ct = guiTitles.containsKey(title);
@@ -332,7 +290,7 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
                 if (!ct) {
                     log.warn(s); // Record in log as well
                 }
-                assertTrue(s, ct);
+                assertTrue(ct, s);
             }
         }
     }
@@ -340,20 +298,22 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
     /*
      * Test GUI elements - run for all components
      */
-    public void GUIComponents1() throws Exception {
-        String name = guiItem.getClass().getName();
+    @ParameterizedTest
+    @MethodSource("guiComponents")
+    public void GUIComponents1(GuiComponentHolder componentHolder) throws Exception {
+        JMeterGUIComponent guiItem = componentHolder.getComponent();
+        String name = componentHolder.toString();
 
-        assertEquals("Name should be same as static label for " + name, guiItem.getStaticLabel(), guiItem.getName());
-        if (name.startsWith("org.apache.jmeter.examples.")){
+        if (guiItem.getClass().getName().startsWith("org.apache.jmeter.examples.")){
             return;
         }
-        if (!name.endsWith("TestBeanGUI")) {
+        if (guiItem.getClass() != TestBeanGUI.class) {
             try {
                 String label = guiItem.getLabelResource();
-                assertNotNull("Label should not be null for "+name, label);
-                assertTrue("Label should not be empty for "+name, !label.isEmpty());
-                assertFalse("'" + label + "' should be in resource file for " + name, JMeterUtils.getResString(
-                        label).startsWith(JMeterUtils.RES_KEY_PFX));
+                assertNotNull(label, () -> "Label should not be null for " + name);
+                assertFalse(label.isEmpty(), () -> "Label should not be empty for " + name);
+                assertFalse(JMeterUtils.getResString(
+                        label).startsWith(JMeterUtils.RES_KEY_PFX), () -> "'" + label + "' should be in resource file for " + name);
             } catch (UnsupportedOperationException uoe) {
                 log.warn("Class has not yet implemented getLabelResource {}", name);
             }
@@ -364,23 +324,32 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
     /*
      * Test GUI elements - not run for TestBeanGui items
      */
-    public void GUIComponents2() throws Exception {
+    @ParameterizedTest
+    @MethodSource("guiComponents")
+    public void GUIComponents2(GuiComponentHolder componentHolder) throws Exception {
+        JMeterGUIComponent guiItem = componentHolder.getComponent();
         String name = guiItem.getClass().getName();
 
         // TODO these assertions should be separate tests
 
         TestElement el = guiItem.createTestElement();
-        assertNotNull(name + ".createTestElement should be non-null ", el);
-        assertEquals("GUI-CLASS: Failed on " + name, name, el.getPropertyAsString(TestElement.GUI_CLASS));
+        assertNotNull(el, name + ".createTestElement should be non-null ");
+        assertEquals(name, el.getPropertyAsString(TestElement.GUI_CLASS), "GUI-CLASS: Failed on " + name);
 
-        assertEquals("NAME: Failed on " + name, guiItem.getName(), el.getName());
-        assertEquals("TEST-CLASS: Failed on " + name, el.getClass().getName(), el
-                .getPropertyAsString(TestElement.TEST_CLASS));
+        assertEquals(guiItem.getName(), el.getName(), () -> "NAME: Failed on " + name);
+        if (StringUtils.isEmpty(el.getName())) {
+            fail("Name of the element must not be blank. Gui class " + name + ", element class " + el.getClass().getName());
+        }
+        assertEquals(el.getClass().getName(), el
+                .getPropertyAsString(TestElement.TEST_CLASS), "TEST-CLASS: Failed on " + name);
+        if (guiItem.getClass() != TestFragmentControllerGui.class) {
+            assertTrue(el.isEnabled(), "Should be enabled by default: " + name);
+        }
         TestElement el2 = guiItem.createTestElement();
         el.setName("hey, new name!:");
         el.setProperty("NOT", "Shouldn't be here");
         if (!(guiItem instanceof UnsharedComponent)) {
-            assertEquals("SHARED: Failed on " + name, "", el2.getPropertyAsString("NOT"));
+            assertEquals("", el2.getPropertyAsString("NOT"), () -> "SHARED: Failed on " + name);
         }
         log.debug("Saving element: {}", el.getClass());
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
@@ -389,95 +358,238 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
         bos.close();
         el = (TestElement) SaveService.loadElement(bis);
         bis.close();
-        assertNotNull("Load element failed on: "+name,el);
+        assertNotNull(el, "Load element failed on: "+name);
         guiItem.configure(el);
-        assertEquals("CONFIGURE-TEST: Failed on " + name, el.getName(), guiItem.getName());
+        assertEquals(el.getName(), guiItem.getName(), () -> "CONFIGURE-TEST: Failed on " + name);
         guiItem.modifyTestElement(el2);
-        assertEquals("Modify Test: Failed on " + name, "hey, new name!:", el2.getName());
+        assertEquals("hey, new name!:", el2.getName(), () -> "Modify Test: Failed on " + name);
     }
 
-    public void propertiesShouldNotBeInitializedToNullValues() {
+    private static final Set<PropertyDescriptor<?, ?>> IGNORED_PROPERTIES = new HashSet<>();
+
+    static {
+        IGNORED_PROPERTIES.add(TestElementSchema.INSTANCE.getGuiClass());
+        IGNORED_PROPERTIES.add(TestElementSchema.INSTANCE.getTestClass());
+        // TODO: support variables in TestElement.enabled property
+        IGNORED_PROPERTIES.add(TestElementSchema.INSTANCE.getEnabled());
+        IGNORED_PROPERTIES.add(AbstractScopedTestElementSchema.INSTANCE.getScope());
+        IGNORED_PROPERTIES.add(ThreadGroupSchema.INSTANCE.getOnSampleError());
+        // TODO: migrate to editable checkboxes
+        IGNORED_PROPERTIES.add(IfControllerSchema.INSTANCE.getEvaluateAll());
+        IGNORED_PROPERTIES.add(IfControllerSchema.INSTANCE.getUseExpression());
+        IGNORED_PROPERTIES.add(HTTPSamplerBaseSchema.INSTANCE.getPostBodyRaw());
+        // TODO: LoopControlPanel does not set continueForever properly
+        IGNORED_PROPERTIES.add(LoopControllerSchema.INSTANCE.getContinueForever());
+        IGNORED_PROPERTIES.add(RegexExtractorSchema.INSTANCE.getMatchTarget());
+        IGNORED_PROPERTIES.add(RegexExtractorSchema.INSTANCE.getDefaultIsEmpty());
+        // TODO: support expressions?
+        IGNORED_PROPERTIES.add(HTTPSamplerBaseSchema.INSTANCE.getIpSourceType());
+        IGNORED_PROPERTIES.add(HTTPSamplerBaseSchema.INSTANCE.getImplementation());
+        // TODO: support expressions in UrlConfigGui
+        IGNORED_PROPERTIES.add(HTTPSamplerBaseSchema.INSTANCE.getFollowRedirects());
+        IGNORED_PROPERTIES.add(HTTPSamplerBaseSchema.INSTANCE.getAutoRedirects());
+
+    }
+
+    /**
+     * Assign simple expression value to every property of the element, and verify if the property is get back correctly
+     * from the UI.
+     */
+    @ParameterizedTest
+    @MethodSource("customGuiComponents")
+    public void allPropertiesAreStoredInUI(GuiComponentHolder componentHolder) {
+        JMeterGUIComponent guiItem = componentHolder.getComponent();
+        assumeFalse(
+                improperlyUsesUiPlaceholders(guiItem.getClass()),
+                () -> "UI " + componentHolder + " does not use placeholders properly, so the test is skipped");
+        assumeFalse(guiItem.getClass() == JMSSamplerGui.class,
+                "JMSSamplerGui does not seem to use default values vs placeholders properly");
+        assumeFalse(guiItem.getClass() == AjpSamplerGui.class,
+                "AjpSamplerGui hides some fields from HTTP (e.g. proxy), so we skip testing AJP");
         TestElement el = guiItem.createTestElement();
+        TestElementSchema schema = el.getSchema();
+        // UI might set properties in a different order which makes it harder to compare
+        Collection<PropertyDescriptor<?, ?>> properties = schema.getProperties().values();
+        for (PropertyDescriptor<?, ?> property : properties) {
+            if (IGNORED_PROPERTIES.contains(property)) {
+                continue;
+            }
+            if (property instanceof CollectionPropertyDescriptor || property instanceof TestElementPropertyDescriptor) {
+                continue;
+            }
+            if (guiItem.getClass() == NamePanel.class && property.equals(TestElementSchema.INSTANCE.getComments())) {
+                // NamePanel does not configure description
+                continue;
+            }
+            if ((guiItem.getClass() == SetupThreadGroupGui.class || guiItem.getClass() == PostThreadGroupGui.class) &&
+                    property.equals(ThreadGroupSchema.INSTANCE.getDelayedStart())) {
+                // Setup and Post thread groups do not show "delay thread creation" checkbox
+                continue;
+            }
+            if (guiItem.getClass() == OpenModelThreadGroupGui.class && (
+                    property.equals(ThreadGroupSchema.INSTANCE.getNumThreads()) ||
+                            property.equals(ThreadGroupSchema.INSTANCE.getSameUserOnNextIteration()))) {
+                continue;
+            }
+            el.set(property, "${test_" + property.getName() + "}");
+        }
+        // Configure UI with the modified properties
+        guiItem.configure(el);
+        // Assign the values from the UI to another element
+        TestElement el2 = guiItem.createTestElement();
+        guiItem.modifyTestElement(el2);
+
+        // Remove all ignored properties
+        for (PropertyDescriptor<?, ?> property : IGNORED_PROPERTIES) {
+            if (property.equals(TestElementSchema.INSTANCE.getGuiClass())) {
+                continue;
+            }
+            el.removeProperty(property);
+            el2.removeProperty(property);
+        }
+        if (guiItem.getClass() == GraphQLHTTPSamplerGui.class) {
+            el.removeProperty(HTTPSamplerBaseSchema.INSTANCE.getArguments());
+            el2.removeProperty(HTTPSamplerBaseSchema.INSTANCE.getArguments());
+            el.removeProperty(HTTPSamplerBaseSchema.INSTANCE.getUseBrowserCompatibleMultipart());
+            el2.removeProperty(HTTPSamplerBaseSchema.INSTANCE.getUseBrowserCompatibleMultipart());
+            el.removeProperty(HTTPSamplerBaseSchema.INSTANCE.getUseMultipartPost());
+            el2.removeProperty(HTTPSamplerBaseSchema.INSTANCE.getUseMultipartPost());
+        }
+
+        compareAllProperties(el, el2,
+                () -> "GUI element " + componentHolder + " be able to pass all the properties to a different TestElement");
+    }
+
+    @ParameterizedTest
+    @MethodSource("guiComponents")
+    public void propertiesShouldNotBeInitializedToNullValues(GuiComponentHolder componentHolder) {
+        JMeterGUIComponent guiItem = componentHolder.getComponent();
+        TestElement el = guiItem.createTestElement();
+
+        assertFalse(
+                StringUtils.isEmpty(el.getName()),
+                () -> "Name should be non-blank for element " + componentHolder);
         PropertyIterator it = el.propertyIterator();
         while (it.hasNext()) {
             JMeterProperty property = it.next();
             if (property.getObjectValue() == null) {
-                fail(
-                        "Property " + property.getName() + " is initialized with NULL OBJECT value in " +
-                                " test element " + el + " created with " + guiItem + ".createTestElement() " +
-                                "Please refrain from that since null properties consume memory, and they will be " +
-                                "removed when saving and loading the plan anyway"
-                );
+                fail("Property " + property.getName() + " is initialized with NULL OBJECT value in " +
+                        " test element " + el + " created with " + guiItem + ".createTestElement() " +
+                        "Please refrain from that since null properties consume memory, and they will be " +
+                        "removed when saving and loading the plan anyway");
             }
             if (property.getStringValue() == null) {
-                fail(
-                        "Property " + property.getName() + " is initialized with NULL STRING value in " +
-                                " test element " + el + " created with " + guiItem + ".createTestElement() " +
-                                "Please refrain from that since null properties consume memory, and they will be " +
-                                "removed when saving and loading the plan anyway"
-                );
+                fail("Property " + property.getName() + " is initialized with NULL STRING value in " +
+                        " test element " + el + " created with " + guiItem + ".createTestElement() " +
+                        "Please refrain from that since null properties consume memory, and they will be " +
+                        "removed when saving and loading the plan anyway");
             }
         }
     }
 
-    public void saveLoadShouldKeepElementIntact() throws IOException {
+    @ParameterizedTest
+    @MethodSource("guiComponents")
+    public void elementShouldNotBeModifiedWithConfigureModify(GuiComponentHolder componentHolder) {
+        JMeterGUIComponent guiItem = componentHolder.getComponent();
+        TestElement expected = guiItem.createTestElement();
+        TestElement actual = guiItem.createTestElement();
+        guiItem.configure(actual);
+        if (!Objects.equals(expected, actual)) {
+            boolean breakpointForDebugging = Objects.equals(expected, actual);
+            String expectedStr = new DslPrinterTraverser(DslPrinterTraverser.DetailLevel.ALL).append(expected).toString();
+            String actualStr = new DslPrinterTraverser(DslPrinterTraverser.DetailLevel.ALL).append(actual).toString();
+            assertEquals(
+                    expectedStr,
+                    actualStr,
+                    () -> "TestElement should not be modified by " + guiItem.getClass().getName() + ".configure(element)"
+            );
+        }
+        guiItem.modifyTestElement(actual);
+        if (guiItem.getClass() == GraphQLHTTPSamplerGui.class) {
+            // GraphQL sampler computes its arguments, so we don't compare them
+            // See org.apache.jmeter.protocol.http.config.gui.GraphQLUrlConfigGui.modifyTestElement
+            expected.removeProperty(HTTPSamplerBaseSchema.INSTANCE.getArguments());
+            actual.removeProperty(HTTPSamplerBaseSchema.INSTANCE.getArguments());
+        }
+        if (!Objects.equals(expected, actual)) {
+            if (improperlyUsesUiPlaceholders(guiItem.getClass())) {
+                return;
+            }
+            boolean breakpointForDebugging = Objects.equals(expected, actual);
+            String expectedStr = new DslPrinterTraverser(DslPrinterTraverser.DetailLevel.ALL).append(expected).toString();
+            String actualStr = new DslPrinterTraverser(DslPrinterTraverser.DetailLevel.ALL).append(actual).toString();
+            assertEquals(
+                    expectedStr,
+                    actualStr,
+                    () -> "TestElement should not be modified by " + guiItem.getClass().getName() + ".configure(element); gui.modifyTestElement(element)"
+            );
+        }
+    }
+
+    private static boolean improperlyUsesUiPlaceholders(Class<? extends JMeterGUIComponent> klass) {
+        if (klass == JavaConfigGui.class || klass == JavaTestSamplerGui.class) {
+            // TODO: JavaConfigGui modifies UI when classname combobox changes, and it causes inconsistency between the
+            //   element state and the UI state. We ignore the discrepancy for now
+            return true;
+        }
+        if (klass == JUnitTestSamplerGui.class) {
+            // TODO: fix org.apache.jmeter.protocol.java.control.gui.JUnitTestSamplerGui.configure to use placeholders
+            return true;
+        }
+        if (klass == BackendListenerGui.class) {
+            // TODO: fix handling of default arguments in org.apache.jmeter.visualizers.backend.BackendListenerGui.actionPerformed
+            return true;
+        }
+        return false;
+    }
+
+    @ParameterizedTest
+    @MethodSource("guiComponents")
+    public void saveLoadShouldKeepElementIntact(GuiComponentHolder componentHolder) throws IOException {
+        JMeterGUIComponent guiItem = componentHolder.getComponent();
         TestElement expected = guiItem.createTestElement();
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         SaveService.saveElement(expected, bos);
         byte[] serializedBytes = bos.toByteArray();
         TestElement actual = (TestElement) SaveService.loadElement(new ByteArrayInputStream(serializedBytes));
-        compareAllProperties(expected, actual, serializedBytes);
+        compareAllProperties(expected, actual,
+                () -> "TestElement after 'save+load' should match the one created in GUI\n" +
+                        "JMX is " + new String(serializedBytes, StandardCharsets.UTF_8));
     }
 
-    private static void compareAllProperties(TestElement expected, TestElement actual, byte[] serializedBytes) {
+    private static void compareAllProperties(TestElement expected, TestElement actual,
+            Supplier<String> message) {
         expected.traverse(IsEnabledNormalizer.INSTANCE);
         actual.traverse(IsEnabledNormalizer.INSTANCE);
 
         String expectedStr = new DslPrinterTraverser(DslPrinterTraverser.DetailLevel.ALL).append(expected).toString();
         if (!Objects.equals(expected, actual)) {
-            boolean abc = Objects.equals(expected, actual);
+            boolean breakpointForDebugging = Objects.equals(expected, actual);
             assertEquals(
-                    "TestElement after 'save+load' should match the one created in GUI\n" +
-                            "JMX is " + new String(serializedBytes, StandardCharsets.UTF_8),
                     expectedStr,
-                    new DslPrinterTraverser(DslPrinterTraverser.DetailLevel.ALL).append(actual).toString()
-            );
-            fail("TestElement after 'save+load' should match the one created in GUI. " +
-                    "DSL representation is the same, however TestElement#equals says the elements are different. " +
-                    "DSL is " + expectedStr + "\n" +
-                    "JMX is " + new String(serializedBytes, StandardCharsets.UTF_8));
+                    new DslPrinterTraverser(DslPrinterTraverser.DetailLevel.ALL).append(actual).toString(),
+                    message.get());
+            fail("DSL representation is the same, however TestElement#equals says the elements are different. " + message.get());
         }
-        assertEquals(
+        assertEquals(expected.hashCode(), actual.hashCode(),
                 "TestElement.hashCode after 'save+load' should match the one created in GUI. " +
-                "DSL representation is the same, however TestElement#hashCode says the elements are different. " +
-                "DSL is " + expectedStr + "\n" +
-                "JMX is " + new String(serializedBytes, StandardCharsets.UTF_8),
-                expected.hashCode(),
-                actual.hashCode()
-        );
+                        "DSL representation is the same, however TestElement#hashCode says the elements are different. " +
+                        message.get());
     }
 
-    /*
-     * Test serializable elements - create the suite of tests
-     */
-    private static Test suiteSerializableElements() throws Throwable {
-        TestSuite suite = new TestSuite("SerializableElements");
-        for (Object o : getObjects(Serializable.class)) {
-            Serializable serObj = (Serializable) o;
-            if (serObj.getClass().getName().endsWith("_Stub")) {
-                continue;
-            }
-            TestSuite ts = new TestSuite(serObj.getClass().getName());
-            ts.addTest(new JMeterTest("runSerialTest", serObj));
-            suite.addTest(ts);
-        }
-        return suite;
+    static Stream<Serializable> serializableObjects() throws Throwable {
+        return getObjects(Serializable.class)
+                .stream()
+                .map(Serializable.class::cast)
+                .filter(o -> !o.getClass().getName().endsWith("_Stub"));
     }
 
     /*
      * Test serializable elements - test the object
      */
-    public void runSerialTest() throws Exception {
+    @ParameterizedTest
+    @MethodSource("serializableObjects")
+    public void runSerialTest(Serializable serObj) throws Exception {
         if (!(serObj instanceof Component)) {//
             try {
                 ByteArrayOutputStream bytes = new ByteArrayOutputStream();
@@ -487,8 +599,10 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
                 ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(bytes.toByteArray()));
                 Object readObject = in.readObject();
                 in.close();
-                assertEquals("deserializing class: " + serObj.getClass().getName(), serObj.getClass(), readObject
-                        .getClass());
+                assertEquals(
+                        serObj.getClass(),
+                        readObject.getClass(),
+                        () -> "deserializing class: " + serObj.getClass().getName());
             } catch (Exception e) {
                 fail("serialization of " + serObj.getClass().getName() + " failed: " + e);
             }
@@ -496,9 +610,10 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
     }
 
 
-    public void readAliases() throws Exception {
+    @BeforeAll
+    public static void readAliases() throws Exception {
         nameMap = SaveService.loadProperties();
-        assertNotNull("SaveService nameMap (saveservice.properties) should not be null",nameMap);
+        assertNotNull(nameMap, "SaveService nameMap (saveservice.properties) should not be null");
     }
 
     private void checkElementAlias(Object item) {
@@ -528,6 +643,11 @@ public class JMeterTest extends JMeterTestCaseJUnit implements Describable {
                 // TODO: Cannot start. travis-job-e984b3d5-f93f-4b0f-b6c0-50988a5ece9d is a loopback address.
                 continue;
             }
+            if (className.startsWith("org.apache.jmeter.testelement.schema.") &&
+                    className.endsWith("PropertyDescriptor")) {
+                // PropertyDescriptors do not have no-arg constructor
+                continue;
+            }
             try {
                 // Construct classes in the AWT thread, as we may have found classes, that
                 // assume to be constructed in the AWT thread.
