@@ -55,8 +55,8 @@ public class MenuScroller {
 
     private JPopupMenu menu;
     private Component[] menuItems;
-    private MenuScrollItem upItem;
-    private MenuScrollItem downItem;
+    private final MenuScrollItem upItem;
+    private final MenuScrollItem downItem;
     private final MenuScrollListener menuListener = new MenuScrollListener();
     private final MouseWheelListener mouseWheelListener = new MouseScrollListener();
     private int scrollCount;
@@ -514,18 +514,6 @@ public class MenuScroller {
         }
     }
 
-    /**
-     * Ensures that the <code>dispose</code> method of this MenuScroller is
-     * called when there are no more references to it.
-     *
-     * @see MenuScroller#dispose()
-     */
-    @Override
-    public void finalize() throws Throwable {
-        dispose();
-        super.finalize();
-    }
-
     private void refreshMenu() {
         if (menuItems != null && menuItems.length > 0 && scrollCount <= menuItems.length) {
             firstIndex = Math.max(topFixedCount, firstIndex);
@@ -668,7 +656,9 @@ public class MenuScroller {
     private enum MenuIcon implements Icon {
 
         UP(9, 1, 9), DOWN(1, 9, 1);
+        @SuppressWarnings("ImmutableEnumChecker")
         final int[] xPoints = { 1, 5, 9 };
+        @SuppressWarnings("ImmutableEnumChecker")
         final int[] yPoints;
 
         MenuIcon(int... yPoints) {
